@@ -6,7 +6,7 @@ use atomic_refcell::AtomicRefCell;
 use glib::{prelude::*, translate::*};
 use gst::{prelude::*, subclass::prelude::*};
 
-use crate::{prelude::BaseSrcExtManual, BaseSrc};
+use crate::{ffi, prelude::*, BaseSrc};
 
 #[derive(Default)]
 pub(super) struct InstanceData {
@@ -20,7 +20,7 @@ pub enum CreateSuccess {
     NewBufferList(gst::BufferList),
 }
 
-pub trait BaseSrcImpl: BaseSrcImplExt + ElementImpl {
+pub trait BaseSrcImpl: ElementImpl + ObjectSubclass<Type: IsA<BaseSrc>> {
     fn start(&self) -> Result<(), gst::ErrorMessage> {
         self.parent_start()
     }
@@ -108,61 +108,7 @@ pub trait BaseSrcImpl: BaseSrcImplExt + ElementImpl {
     }
 }
 
-pub trait BaseSrcImplExt: ObjectSubclass {
-    fn parent_start(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_stop(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_is_seekable(&self) -> bool;
-
-    fn parent_size(&self) -> Option<u64>;
-
-    fn parent_times(
-        &self,
-        buffer: &gst::BufferRef,
-    ) -> (Option<gst::ClockTime>, Option<gst::ClockTime>);
-
-    fn parent_fill(
-        &self,
-        offset: u64,
-        length: u32,
-        buffer: &mut gst::BufferRef,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    fn parent_alloc(&self, offset: u64, length: u32) -> Result<gst::Buffer, gst::FlowError>;
-
-    fn parent_create(
-        &self,
-        offset: u64,
-        buffer: Option<&mut gst::BufferRef>,
-        length: u32,
-    ) -> Result<CreateSuccess, gst::FlowError>;
-
-    fn parent_do_seek(&self, segment: &mut gst::Segment) -> bool;
-
-    fn parent_query(&self, query: &mut gst::QueryRef) -> bool;
-
-    fn parent_event(&self, event: &gst::Event) -> bool;
-
-    fn parent_caps(&self, filter: Option<&gst::Caps>) -> Option<gst::Caps>;
-
-    fn parent_negotiate(&self) -> Result<(), gst::LoggableError>;
-
-    fn parent_set_caps(&self, caps: &gst::Caps) -> Result<(), gst::LoggableError>;
-
-    fn parent_fixate(&self, caps: gst::Caps) -> gst::Caps;
-
-    fn parent_unlock(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_unlock_stop(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_decide_allocation(
-        &self,
-        query: &mut gst::query::Allocation,
-    ) -> Result<(), gst::LoggableError>;
-}
-
-impl<T: BaseSrcImpl> BaseSrcImplExt for T {
+pub trait BaseSrcImplExt: BaseSrcImpl {
     fn parent_start(&self) -> Result<(), gst::ErrorMessage> {
         unsafe {
             let data = Self::type_data();
@@ -357,7 +303,7 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
                     if buffer_ptr.is_null() && pending_buffer_list.is_none() {
                         gst::error!(
                             gst::CAT_RUST,
-                            obj: instance,
+                            obj = instance,
                             "No buffer and no buffer list returned"
                         );
                         return Err(gst::FlowError::Error);
@@ -366,7 +312,7 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
                     if !buffer_ptr.is_null() && pending_buffer_list.is_some() {
                         gst::error!(
                             gst::CAT_RUST,
-                            obj: instance,
+                            obj = instance,
                             "Both buffer and buffer list returned"
                         );
                         return Err(gst::FlowError::Error);
@@ -378,7 +324,7 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
 
                             gst::debug!(
                                 gst::CAT_PERFORMANCE,
-                                obj: instance,
+                                obj = instance,
                                 "Returned new buffer from parent create function, copying into passed buffer"
                             );
 
@@ -387,7 +333,7 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
                                 Err(_) => {
                                     gst::error!(
                                         gst::CAT_RUST,
-                                        obj: instance,
+                                        obj = instance,
                                         "Failed to map passed buffer writable"
                                     );
                                     return Err(gst::FlowError::Error);
@@ -401,12 +347,12 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
                                 passed_buffer.set_size(copied_size);
                             }
 
-                            match new_buffer.copy_into(passed_buffer, gst::BUFFER_COPY_METADATA, 0, None) {
+                            match new_buffer.copy_into(passed_buffer, gst::BUFFER_COPY_METADATA, ..) {
                                 Ok(_) => Ok(CreateSuccess::FilledBuffer),
                                 Err(_) => {
                                     gst::error!(
                                         gst::CAT_RUST,
-                                        obj: instance,
+                                        obj = instance,
                                         "Failed to copy buffer metadata"
                                     );
 
@@ -607,6 +553,8 @@ impl<T: BaseSrcImpl> BaseSrcImplExt for T {
     }
 }
 
+impl<T: BaseSrcImpl> BaseSrcImplExt for T {}
+
 unsafe impl<T: BaseSrcImpl> IsSubclassable<T> for BaseSrc {
     fn class_init(klass: &mut glib::Class<Self>) {
         Self::parent_class_init::<T>(klass);
@@ -799,7 +747,7 @@ unsafe extern "C" fn base_src_create<T: BaseSrcImpl>(
                     if passed_buffer.as_ptr() != new_buffer.as_ptr() {
                         gst::debug!(
                             gst::CAT_PERFORMANCE,
-                            obj: instance,
+                            obj = instance,
                             "Returned new buffer from create function, copying into passed buffer"
                         );
 
@@ -808,7 +756,7 @@ unsafe extern "C" fn base_src_create<T: BaseSrcImpl>(
                             Err(_) => {
                                 gst::error!(
                                     gst::CAT_RUST,
-                                    obj: instance,
+                                    obj = instance,
                                     "Failed to map passed buffer writable"
                                 );
                                 return gst::FlowReturn::Error;
@@ -822,17 +770,12 @@ unsafe extern "C" fn base_src_create<T: BaseSrcImpl>(
                             passed_buffer.set_size(copied_size);
                         }
 
-                        match new_buffer.copy_into(
-                            passed_buffer,
-                            gst::BUFFER_COPY_METADATA,
-                            0,
-                            None,
-                        ) {
+                        match new_buffer.copy_into(passed_buffer, gst::BUFFER_COPY_METADATA, ..) {
                             Ok(_) => gst::FlowReturn::Ok,
                             Err(_) => {
                                 gst::error!(
                                     gst::CAT_RUST,
-                                    obj: instance,
+                                    obj = instance,
                                     "Failed to copy buffer metadata"
                                 );
 
