@@ -14,7 +14,7 @@ pub enum BaseTransformMode {
     Both,
 }
 
-pub trait BaseTransformImpl: BaseTransformImplExt + ElementImpl {
+pub trait BaseTransformImpl: ElementImpl + ObjectSubclass<Type: IsA<BaseTransform>> {
     const MODE: BaseTransformMode;
     const PASSTHROUGH_ON_SAME_CAPS: bool;
     const TRANSFORM_IP_ON_PASSTHROUGH: bool;
@@ -154,12 +154,7 @@ pub trait BaseTransformImpl: BaseTransformImplExt + ElementImpl {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::BaseTransformImplExt> Sealed for T {}
-}
-
-pub trait BaseTransformImplExt: sealed::Sealed + ObjectSubclass {
+pub trait BaseTransformImplExt: BaseTransformImpl {
     fn parent_start(&self) -> Result<(), gst::ErrorMessage> {
         unsafe {
             let data = Self::type_data();
@@ -374,9 +369,7 @@ pub trait BaseTransformImplExt: sealed::Sealed + ObjectSubclass {
                         "transform doesn't operate in-place"
                     ))
                 } else {
-                    unreachable!(concat!(
-                        "parent `get_unit_size` called while transform operates in-place"
-                    ))
+                    unreachable!("parent `get_unit_size` called while transform operates in-place")
                 }
             });
 
@@ -459,18 +452,16 @@ pub trait BaseTransformImplExt: sealed::Sealed + ObjectSubclass {
                         (&mut outbuf) as *mut *mut gst::ffi::GstBuffer as *mut gst::ffi::GstBuffer,
                     ))
                     .map(|_| {
-                        if outbuf == buf as *mut _ {
+                        if ptr::eq(outbuf, buf as *mut _) {
                             PrepareOutputBufferSuccess::InputBuffer
                         } else {
                             PrepareOutputBufferSuccess::Buffer(from_glib_full(outbuf))
                         }
                     })
-                    .map_err(|err| {
-                        if outbuf != buf as *mut _ {
+                    .inspect_err(|_err| {
+                        if !ptr::eq(outbuf, buf as *mut _) {
                             drop(Option::<gst::Buffer>::from_glib_full(outbuf));
                         }
-
-                        err
                     })
                 })
                 .unwrap_or(Err(gst::FlowError::NotSupported))
@@ -501,9 +492,7 @@ pub trait BaseTransformImplExt: sealed::Sealed + ObjectSubclass {
                     if !self.obj().unsafe_cast_ref::<BaseTransform>().is_in_place() {
                         Err(gst::FlowError::NotSupported)
                     } else {
-                        unreachable!(concat!(
-                            "parent `transform` called while transform operates in-place"
-                        ));
+                        unreachable!("parent `transform` called while transform operates in-place");
                     }
                 })
         }
@@ -523,9 +512,9 @@ pub trait BaseTransformImplExt: sealed::Sealed + ObjectSubclass {
                         "transform operates in-place"
                     ));
                 } else {
-                    unreachable!(concat!(
+                    unreachable!(
                         "parent `transform` called while transform doesn't operate in-place"
-                    ));
+                    );
                 }
             });
 
@@ -1173,7 +1162,7 @@ unsafe extern "C" fn base_transform_propose_allocation<T: BaseTransformImpl>(
         match imp.propose_allocation(decide_query, query) {
             Ok(()) => true,
             Err(err) => {
-                err.log_with_imp(imp);
+                err.log_with_imp_and_level(imp, gst::DebugLevel::Info);
                 false
             }
         }
